# 3d wireframe consists of
# Nodes: points with three coordinates x,y,z
# Edges (or vertices): lines connecting two nodes
# Faces: surfaces bounded by multiple edges

#we will create wireframe objects (only nodes + edges)
#we wont deal with shading/texture


#We will define a cube
#Then we will project it.
#Project it is to convert 3D coordinates (x,y,z) to 2D coordinates (screen_x, screen_y)
#==> This is called projection
# many types of projection=> one is to look parallel with the depth axis (z)
#
# we will use a linear trasnform T(x,y,z)-> (x,y)

#Session5
#for rotations we need an angle and a vector
#it is easier to have polar coordinates (angle and distance from the point of rotation)
#(x,y)=> (theta, d)
#angle theta in radians using atan2()

import math



class Node:
    def __init__(self,coordinates):
        self.x = coordinates[0]
        self.y = coordinates[1]
        self.z = coordinates[2]

class Edge:
    def __init__(self, start, stop):
        self.start = start
        self.stop = stop

class Wireframe:
    def __init__(self):
        self.nodes = []
        self.edges = []

    #take a list of coordinates
    def addNodes(self, nodeList):
        for node in nodeList:
            self.nodes.append(Node(node))

    #take a list of pairs of node indices to join
    def addEdges(self, edgeList):
        for (start,stop) in edgeList:
            self.edges.append(Edge(self.nodes[start], self.nodes[stop]))


#Also add output functions for debugging
    def outputNodes(self):
        print("\n --- Nodes ---")
        for i, node in enumerate(self.nodes):
            print("%d: (%.2f, %.2f, %.2f)" % (i, node.x, node.y, node.z))

    def outputEdges(self):
        print("\n --- Edges ---")
        for i, edge in enumerate(self.edges):
            print("%d: (%.2f, %.2f, %.2f)" % (i, edge.start.x, edge.start.y, edge.start.z))
            print("--> (%.2f, %.2f, %.2f)" % ( edge.stop.x, edge.stop.y, edge.stop.z))

    #function to move along an axis
    #usage: cube.translate('x',100)
    def translate(self, axis,d):
        if axis in ['x', 'y', 'z']:
            for node in self.nodes:
                setattr(node, axis, getattr(node, axis)+d)

    #we scale from the center, the nodes approach the given center
    #we ignore the z coordinate
    #usage: cube.scale((200,150),0.75)
    def scale(self, center_xcenter_y, scale):
        center_x,center_y=center_xcenter_y
        for node in self.nodes:
            node.x = center_x + scale * (node.x - center_x)
            node.y = center_y + scale * (node.y - center_y)
            node.z *= scale

    #Session5
    def findCenter(self):
        num_nodes = len(self.nodes)
        meanX = sum([node.x for node in self.nodes]) / num_nodes
        meanY = sum([node.y for node in self.nodes]) / num_nodes
        meanZ = sum([node.z for node in self.nodes]) / num_nodes
        return (meanX, meanY, meanZ)

    #Session5
    def rotateZ(self, cxcycz, radians):
        cx,cy,cz=cxcycz
        for node in self.nodes:
            x = node.x - cx
            y = node.y - cy
            d = math.hypot(y,x)
            theta = math.atan2(y,x)+radians
            node.x = cx + d*math.cos(theta)
            node.y = cy + d*math.sin(theta)

    def rotateX(self, cxcycz, radians):
        cx,cy,cz=cxcycz
        for node in self.nodes:
            y = node.y - cy
            z = node.z - cz
            d = math.hypot(y,z)
            theta = math.atan2(y,z) + radians
            node.z = cz + d * math.cos(theta)
            node.y = cy + d * math.sin(theta)

    def rotateY(self, cxcycz, radians):
        cx,cy,cz=cxcycz
        for node in self.nodes:
            x = node.x -cx
            z = node.z - cz
            d = math.hypot(x,z)
            theta = math.atan2(x,z) + radians
            node.z = cz + d*math.cos(theta)
            node.x = cx + d*math.sin(theta)




if __name__ == "__main__":
    #we define from 0,0,0 every node up to 1,1,1
    cube_nodes = [(x,y,z) for x in (0,1) for y in (0,1) for z in (0,1)]
    cube = Wireframe()
    cube.addNodes(cube_nodes)

    #parallel to the x-axis edges. Front: 0-2-6-4 back 1-3-7-5
    #(0,4), (1,5), (2,6), (3,7)
    cube.addEdges([(n,n+4) for n in range(0,4)])

    cube.addEdges([(n,n+1) for n in range(0,8,2)])
    cube.addEdges([(n,n+2) for n in (0,1,4,5)])
    cube.outputNodes()
    cube.outputEdges()

    
